@@ -3,6 +3,7 @@
 
 // create an RNG and then seed it, or create it using a state constructed from somewhere else
 
+#include <stdbool.h>
 #include "stddef.h"
 #include <stdint.h>
 
@@ -34,16 +35,36 @@ typedef enum {
     EXT_RNG_STANDARD_NORMAL_INVALID // must be last
 } ext_rng_standardNormal_t;
 
-
 // state can be null; size is determined by algorithm; see below for a few state structs
 ext_rng* ext_rng_create(ext_rng_algorithm_t algorithm, const void* state);
 void ext_rng_destroy(ext_rng* generator);
+
+void ext_rng_setState(ext_rng* generator, const void* state);
+
+// createDefault seeds the result so it is ready to use
+// useNative attempts to use the rng in the embedded environment; generally not thread safe
+ext_rng* ext_rng_createDefault(bool useNative);
+
+int ext_rng_createAndSeed(ext_rng** result, ext_rng_algorithm_t algorithm, ext_rng_standardNormal_t standardNormalAlgorithm);
+
+// returns what will be used in createDefault, unless useNative is specified;
+ext_rng_algorithm_t ext_rng_getDefaultAlgorithmType();
+ext_rng_standardNormal_t ext_rng_getDefaultStandardNormalType();
+
 // state can be null; for BOX_MULLER, it should point to a double that is the next number, or 0.0 if that isn't set yet
 // for USER_NORM, it should be a userFunction outlined below
 int ext_rng_setStandardNormalAlgorithm(ext_rng* generator, ext_rng_standardNormal_t standardNormalAlgorithm, const void* state);
 int ext_rng_setSeed(ext_rng* generator, uint_least32_t seed);
 int ext_rng_setSeedFromClock(ext_rng* generator);
 
+// not the same as "state" above, since it also includes the status of the
+// standard normal algorithm and gamma simulation;
+// guarantees that the result is aligned to sizeof(int), however the length is in characters
+// user is responsible for seralizing user functions in algorithm or standardNormalAlgorithm
+ext_size_t ext_rng_getSerializedStateLength(const ext_rng* generator);
+void ext_rng_writeSerializedState(const ext_rng* generator, void* state);
+void ext_rng_readSerializedState(ext_rng* generator, const void* state);
+
 double ext_rng_simulateContinuousUniform(ext_rng* generator); // randomBase.c
 double ext_rng_simulateStandardNormal(ext_rng* generator);    // randomNorm.c
 
@@ -102,3 +123,4 @@ typedef struct {
 #endif
 
 #endif // EXTERNAL_RANDOM_H
+
